@@ -15,7 +15,6 @@ import mdformat
 from mdformat._conf import DEFAULT_OPTS, InvalidConfError, read_toml_opts
 from mdformat._util import detect_newline_type, is_md_equal
 import mdformat.plugins
-import mdformat.renderer
 
 
 class RendererWarningPrinter(logging.Handler):
@@ -24,7 +23,7 @@ class RendererWarningPrinter(logging.Handler):
             sys.stderr.write(f"Warning: {record.msg}\n")
 
 
-def run(cli_args: Sequence[str]) -> int:  # noqa: C901
+def run(cli_args: Sequence[str], cache_toml: bool = True) -> int:  # noqa: C901
     arg_parser = make_arg_parser(
         mdformat.plugins._PARSER_EXTENSION_DISTS,
         mdformat.plugins._CODEFORMATTER_DISTS,
@@ -47,13 +46,18 @@ def run(cli_args: Sequence[str]) -> int:  # noqa: C901
     format_errors_found = False
     renderer_warning_printer = RendererWarningPrinter()
     for path in file_paths:
+        read_toml = read_toml_opts if cache_toml else read_toml_opts.__wrapped__
         try:
-            toml_opts, toml_path = read_toml_opts(path.parent if path else Path.cwd())
+            toml_opts, toml_path = read_toml(path.parent if path else Path.cwd())
         except InvalidConfError as e:
             print_error(str(e))
             return 1
 
-        opts: Mapping = {**DEFAULT_OPTS, **toml_opts, **cli_core_opts}
+        opts = {**DEFAULT_OPTS, **toml_opts, **cli_core_opts}
+
+        # Merge plugin options from CLI.
+        # Make a copy of opts["plugin"] to not mutate DEFAULT_OPTS or cached TOML.
+        opts["plugin"] = dict(opts["plugin"])
         for plugin_id, plugin_opts in cli_plugin_opts.items():
             if plugin_id in opts["plugin"]:
                 opts["plugin"][plugin_id] |= plugin_opts
@@ -122,13 +126,16 @@ def run(cli_args: Sequence[str]) -> int:  # noqa: C901
             path_str = "-"
             original_str = sys.stdin.read()
 
+        # Lazy import to improve module import time
+        from mdformat.renderer import LOGGER as RENDERER_LOGGER
+
         formatted_str = mdformat.text(
             original_str,
             options=opts,
             extensions=enabled_parserplugins,
             codeformatters=enabled_codeformatters,
             _first_pass_contextmanager=log_handler_applied(
-                mdformat.renderer.LOGGER, renderer_warning_printer
+                RENDERER_LOGGER, renderer_warning_printer
             ),
             _filename=path_str,
         )
